
#include "dcrpgsprite.h"
#include "dcrpggame.h"
#include <kos.h>

using namespace crabclaw;

namespace dcrpg	{
	DCRPGSprite::DCRPGSprite(ccTexture *tex, float u, float v, uint8 list) : ccSprite(tex, u, v, list)	{
		left = NULL;
		right = NULL;
		up = NULL;
		down = NULL;
		runleft = NULL;
		runright = NULL;
		runup = NULL;
		rundown = NULL;
		attackdown = NULL;
		attackup = NULL;
		attackleft = NULL;
		attackright = NULL;
	}
	
	DCRPGSprite::~DCRPGSprite()	{
		delete left;
		delete right;
		delete up;
		delete down;
		delete runleft;
		delete runright;
		delete runup;
		delete rundown;
		delete attackup;
		delete attackdown;
		delete attackleft;
		delete attackright;
	}		
		
	void DCRPGSprite::setAnims(ccSpriteAnim *l, ccSpriteAnim *r, ccSpriteAnim *u, ccSpriteAnim *d,
	                           ccSpriteAnim *rl, ccSpriteAnim *rr, ccSpriteAnim *ru, ccSpriteAnim *rd,
	                           ccSpriteAnim *al, ccSpriteAnim *ar, ccSpriteAnim *au, ccSpriteAnim *ad)	{
		left = l;
		right = r;
		up = u;
		down = d;
		runleft = rl;
		runright = rr;
		runup = ru;
		rundown = rd;
		attackdown = ad;
		attackup = au;
		attackleft = al;
		attackright = ar;
	}
	
	void DCRPGSprite::setDirection(int d)	{
		if(d != 4)
			direction = d;

		// reset anims.
		if (state == STATE_STANDING)	{
			switch(direction)	{
				case FACING_LEFT:
					if (attacking)
						setAnim(attackleft);
					else
						setAnim(left);
					break;
				case FACING_RIGHT:
					if (attacking)
						setAnim(attackright);
					else
						setAnim(right);
					break;
				case FACING_UP:
					if (attacking)
						setAnim(attackup);
					else
						setAnim(up);
					break;
				case FACING_DOWN:
					if (attacking)
						setAnim(attackdown);
					else
						setAnim(down);
					break;
			}	
		}
		else if (state == STATE_WALKING)	{
			switch(direction)	{
				case FACING_LEFT:
					if (running)
						setAnim(runleft);
					else
						setAnim(left);
					break;
				case FACING_RIGHT:
					if (running)
						setAnim(runright);
					else
						setAnim(right);
					break;
				case FACING_UP:
					if (running)
						setAnim(runup);
					else
						setAnim(up);
					break;
				case FACING_DOWN:
					if (running)
						setAnim(rundown);
					else
						setAnim(down);
					break;
			}
		}
		rewindAnimation();
	}
	
	void DCRPGSprite::setState(int s)	{
		state = s;
	}
	
	void DCRPGSprite::setRunning(bool r)	{
		running = r;
	}
	
	void DCRPGSprite::setAttacking(bool a)	{
		attacking = a;
	}
	
	int DCRPGSprite::getDirection()	{
		return direction;
	}
	
	int DCRPGSprite::getState()	{
		return state;
	}
	
	bool DCRPGSprite::isRunning()	{
		return running;
	}
	
	bool DCRPGSprite::isAttacking()	{
		return attacking;
	}
	
	void DCRPGSprite::draw(ccScreen *scr)	{
		incrementAnim();
		if(ccSprite::getXPos() < 640 && ccSprite::getYPos() < 480)
			ccSprite::draw(scr);
	}
}
