/*	main.cc
	DCRPG startup code.
	Copyright (C) 2004 Lawrence Sebald	*/
	
/*	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <ccvideo.h>
#include <ccfont.h>
#include <ccmenu.h>
#include <cctimer.h>
#include <ccsprite.h>
#include <ccvertex.h>
#include <cctexture.h>
#include <cccontroller.h>

#include <iostream>

#include <cstdlib>

#include "dcrpggame.h"
#include "dcrpgsprite.h"

using namespace crabclaw;
using namespace dcrpg;
using namespace std;

DCRPG *dcrpgGame;

static void atexit_handler()	{
	delete dcrpgGame;
}

static void discl(ccScreen *scr, ccController *c1, ccTTFFont *font)	{
	const char discl_msg[] = "DCRPG comes with ABSOLUTELY NO WARRANTY\n"
	                         "and is freely distributed under the terms of the\n" 
	                         "GNU General Public license. See the file COPYING\n"
	                         "for more information.\n\n"
	                         "ljsdcdev and DCRPG are in no way associated\n"
	                         "with or endorsed by Sega.\n\n"
	                         "DCRPG is Copyright (C) 2004 Lawrence Sebald\n"
	                         "Copyright (C) 2004 Kris McAulay\n"
	                         "Visit http://ljsdcdev.sourceforge.net/\n"
	                         "for more information.\n\n"
	                         "Press Start to continue.";

	ccSprite *sbg = new ccSprite(ccTexture::createFromPNG("/cd/disclbg.png"), 1.0f, 1.0f, ccScreen::LIST_OPAQUE);
	bool start_pressed = false;
	sbg->setDimensions(640, 480);
	sbg->setDisplayed(0, 0);
	sbg->setPos(0, 0, 1);
	
	while(!start_pressed)	{
		c1->poll();
		start_pressed = c1->isPressed(3);
		
		scr->beginScene();
		scr->beginList(ccScreen::LIST_OPAQUE);
		sbg->draw(scr);
		scr->finishList();
		scr->beginList(ccScreen::LIST_TRANSLUCENT);
		font->drawString(scr, discl_msg, 32, 64, 26.0f);
		scr->finishList();
		scr->finishScene();
	}
	
	/* deadwait a bit.... */
	uint64 t = ccTimer::getMilliseconds();
	while(t + 300 > ccTimer::getMilliseconds())	;
	
	delete sbg;
}

static ccSelectorMenu *sm;

static void menu_eh(int e, void *param)	{
	uint64 *ts = (uint64 *) param;
	ccController *c1 = dcrpgGame->getController();
	
	c1->poll();
	
	if(c1->isPressed(3))	{
		sm->stop();
	}
	
	if(c1->isPressed(4) && *ts + 200 < ccTimer::getMilliseconds())	{
		if(e != 0)
			sm->decrementSelected();
		else
			sm->setSelected(3);
		*ts = ccTimer::getMilliseconds();
	}
	
	if(c1->isPressed(5) && *ts + 200 < ccTimer::getMilliseconds())	{
		if(e != 3)
			sm->incrementSelected();
		else
			sm->setSelected(0);
		*ts = ccTimer::getMilliseconds();
	}
	
	if(c1->isPressed(2) && *ts + 200 < ccTimer::getMilliseconds())	{
		if(e == 0)	{
			*ts = 0;
			sm->stop();
		}		
	}
}	

static void mainmenu()	{
	ccMovingPoly *mp = new ccMovingPoly(ccScreen::LIST_TRANSLUCENT, NULL);
	uint64 timer = ccTimer::getMilliseconds();
	sm = new ccSelectorMenu(dcrpgGame->getScreen(), mp, 0, 240, 240, 0, 24, menu_eh, &timer);
	bool start_pressed = false;
	uint8 alpha = 0xFF;
	uint8 direction = 0;
	ccScreen *scr = dcrpgGame->getScreen();
	ccTTFFont *font = dcrpgGame->getFont();
	ccController *c1 = dcrpgGame->getController();
	ccTextString *s1 = new ccTextString(font, "New Game", 24.0f, 0xFFFFFFFF, true);
	ccTextString *s2 = new ccTextString(font, "Continue", 24.0f, 0xFF808080, true);
	ccTextString *s3 = new ccTextString(font, "Options", 24.0f, 0xFF808080, true);
	ccTextString *s4 = new ccTextString(font, "Website", 24.0f, 0xFF808080, true);
	
	s1->setPosition(-1, 264);
	s2->setPosition(-1, 288);
	s3->setPosition(-1, 312);
	s4->setPosition(-1, 336);
	
	ccSprite *sbg = new ccSprite(ccTexture::createFromPNG("/cd/menubg.png"), 1.0f, 1.0f, ccScreen::LIST_OPAQUE);
	sbg->setDimensions(640, 480);
	sbg->setDisplayed(0, 0);
	sbg->setPos(0, 0, 1);
	
	mp->addVertex(new ccVertex(ccVertex::NORMAL_VERT, 240, 256, 3, 0.0f, 0.0f, 0x800000FF));
	mp->addVertex(new ccVertex(ccVertex::NORMAL_VERT, 240, 240, 3, 0.0f, 0.0f, 0x800000FF));
	mp->addVertex(new ccVertex(ccVertex::NORMAL_VERT, 400, 256, 3, 0.0f, 0.0f, 0x800000FF));
	mp->addVertex(new ccVertex(ccVertex::END_VERT, 400, 240, 3, 0.0f, 0.0f, 0x800000FF));
	
	sm->add(sbg);
	sm->add(s1);
	sm->add(s2);
	sm->add(s3);
	sm->add(s4);
		
	while(!start_pressed)	{
		if(direction == 0 && timer + 16 < ccTimer::getMilliseconds())	{
			alpha -= 0x01;
			if(alpha == 0x2F)	{
				direction = 1;
			}
			timer = ccTimer::getMilliseconds();
		}
		else if(direction == 1 && timer + 16 < ccTimer::getMilliseconds())	{
			alpha += 0x01;
			if(alpha == 0xFF)	{
				direction = 0;
			}
			timer = ccTimer::getMilliseconds();
		}
		
		c1->poll();
		start_pressed = c1->isPressed(3);
		
		scr->beginScene();
		scr->beginList(ccScreen::LIST_OPAQUE);
		sbg->draw(scr);
		scr->finishList();
		scr->beginList(ccScreen::LIST_TRANSLUCENT);
		font->drawStringCentered(scr, "Press Start Button", 0, 640, 264, 24.0f, ((int) alpha << 24) | 0xFFFFFF);
		scr->finishList();
		scr->finishScene();
	}
	
	/* deadwait a bit.... */
	uint64 t = ccTimer::getMilliseconds();
	while(t + 300 > ccTimer::getMilliseconds())	;

	sm->run();
	
	delete sm;
	
	if(timer == 0)	{
		dcrpgGame->newGame();
	}
}

int main(int argc, char *argv[])	{
	ccScreen *scr;
	ccController *c1;
	ccTTFFont *font;
	
	scr = new ccScreen(0, 0, 640, 480, 16, "DCRPG");
	c1 = new ccController(0);
	font = new ccTTFFont("/cd/bluehigh.ttf", 32);
	
	dcrpgGame = new DCRPG(scr, c1, font);
		
	atexit(atexit_handler);	
		
	discl(scr, c1, font);
	
	mainmenu();
	
	exit(0);
}
